<?php
/*!
 * AJAX functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage AJAX
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the AJAX functionality.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import_Ajax extends Directly_Import_Wrapper
{
	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
		
		add_action('wp_ajax_diim_add_site', array($this, 'add_site'));
		add_action('wp_ajax_diim_archive_site', array($this, 'archive_site'));
		add_action('wp_ajax_diim_delete_site', array($this, 'delete_site'));
		add_action('wp_ajax_diim_import_action', array($this, 'import_action'));
		add_action('wp_ajax_diim_regenerate_api_key', array($this, 'regenerate_api_key'));
		add_action('wp_ajax_diim_restore_site', array($this, 'restore_site'));
	}
	
	/**
	 * Get a default value based on the provided name.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 * @param  string $name Name of the value to return.
	 * @return mixed        Default value if it exists, otherwise an empty string.
	 */
	protected function _default($name)
	{
		switch ($name)
		{
			/**
			 * Post meta keys containing HTML content.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML:
			
				/**
				 * Filters the post meta that contain HTML content.
				 *
				 * @since 1.0.0
				 *
				 * @param  array $keys Meta keys for values that contain HTML content.
				 * @return array       Modified array of meta keys.
				 */
				return apply_filters(Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML, array());
				
			/**
			 * Post meta keys containing post URLs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS:
			
				/**
				 * Filters the post meta that contain post URLs.
				 *
				 * @since 1.0.0
				 *
				 * @param  array $keys Meta keys for values that contain post URLs.
				 * @return array       Modified array of meta keys.
				 */
				return apply_filters(Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS, array());
				
			/**
			 * Post meta keys and prefixes that should be ignored.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_IGNORE:
			
				/**
				 * Filters the post meta keys and prefixes that should be ignored.
				 *
				 * @since 1.0.0
				 *
				 * @param  array $keys_and_prefixes Meta keys and prefixes that should be ignored.
				 * @return array                    Modified array of meta keys.
				 */
				return apply_filters(Directly_Import_Constants::HOOK_POST_META_IGNORE, array('_edit_lock', '_oembed_', '_wp_attachment_backup_sizes', '_wp_attachment_metadata'));
				
			/**
			 * Post meta keys containing post IDs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_POST_IDS:
			
				/**
				 * Filters the post meta that contain post IDs.
				 *
				 * @since 1.0.0
				 *
				 * @param  array $keys Meta keys for values that contain post IDs.
				 * @return array       Modified array of meta keys.
				 */
				return apply_filters(Directly_Import_Constants::HOOK_POST_META_POST_IDS, array('_thumbnail_id'));
				
			/**
			 * Post meta keys containing term IDs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_TERM_IDS:
			
				/**
				 * Filters the post meta that contain term IDs.
				 *
				 * @since 1.0.0
				 *
				 * @param  array $keys Meta keys for values that contain term IDs.
				 * @return array       Modified array of meta keys.
				 */
				return apply_filters(Directly_Import_Constants::HOOK_POST_META_TERM_IDS, array());
				
			/**
			 * Post meta keys containing user IDs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_USER_IDS:
			
				/**
				 * Filters the post meta that contain user IDs.
				 *
				 * @since 1.0.0
				 *
				 * @param  array $keys Meta keys for values that contain user IDs.
				 * @return array       Modified array of meta keys.
				 */
				return apply_filters(Directly_Import_Constants::HOOK_POST_META_USER_IDS, array('_edit_last'));
		}
		
		return parent::_default($name);
	}
	
	/**
	 * Add an import site.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function add_site()
	{
		$url = (isset($_POST['url']))
		? esc_url_raw($_POST['url'])
		: '';
		
		$response = Directly_Import_Utilities::get_response($url);
		
		if ($response['success'])
		{
			$sites = Directly_Import_Utilities::check_array(get_option(Directly_Import_Constants::OPTION_SITES));
			$url_split = explode(Directly_Import_Constants::TOKEN, $url);
			$url_split = explode('?', $url_split[0]);
			$url_split = explode('index.php', $url_split[0]);
			$url_base = trailingslashit($url_split[0]);
			
			if (isset($sites[$url_base]))
			{
				$sites[$url_base]['api_url'] = $url;
				$sites[$url_base]['archived'] = false;
				$sites[$url_base]['available'] = array();
				$sites[$url_base]['home_url'] = $response['home_url'];
				$sites[$url_base]['imported'] = array();
				$sites[$url_base]['uploads_url'] = $response['uploads_url'];
			}
			else
			{
				$sites[$url_base] = array
				(
					'api_url' => $url,
					'archived' => false,
					'available' => array(),
					'home_url' => $response['home_url'],
					'imported' => array(),
					'uploads_url' => $response['uploads_url']
				);
			}

			update_option(Directly_Import_Constants::OPTION_SITES, $sites);
		}
		
		wp_send_json($response);
	}
	
	/**
	 * Archive an import site.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function archive_site()
	{
		$url = (isset($_POST['url']))
		? esc_url_raw($_POST['url'])
		: '';
		
		$sites = Directly_Import_Utilities::check_array(get_option(Directly_Import_Constants::OPTION_SITES));
		
		if
		(
			!empty($url)
			&&
			isset($sites[$url])
		)
		{
			$sites[$url]['archived'] = true;
			
			update_option(Directly_Import_Constants::OPTION_SITES, $sites);
		}
		
		wp_send_json(array
		(
			'success' => true
		));
	}
	
	/**
	 * Delete an import site.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function delete_site()
	{
		$sites = Directly_Import_Utilities::check_array(get_option(Directly_Import_Constants::OPTION_SITES));
		
		$url = (isset($_POST['url']))
		? esc_url_raw($_POST['url'])
		: '';
		
		if
		(
			!empty($url)
			&&
			isset($sites[$url])
		)
		{
			unset($sites[$url]);
			
			update_option(Directly_Import_Constants::OPTION_SITES, $sites);
		}
		
		wp_send_json(array
		(
			'success' => true
		));
	}
	
	/**
	 * Execute and import action.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function import_action()
	{
		$output = array
		(
			'success' => false
		);
		
		$import_action = (isset($_POST['import_action']))
		? sanitize_key($_POST['import_action'])
		: '';
		
		$url = (isset($_POST['url']))
		? esc_url_raw($_POST['url'])
		: '';
		
		$sites = Directly_Import_Utilities::check_array(get_option(Directly_Import_Constants::OPTION_SITES));
		
		if
		(
			!empty($import_action)
			&&
			!empty($url)
			&&
			isset($sites[$url])
		)
		{
			set_time_limit(0);
			
			$args = array
			(
				'import_action' => $import_action
			);
			
			$base_post_type = (isset($_POST['post_type']))
			? sanitize_key($_POST['post_type'])
			: '';
			
			if (!isset($sites[$url]['imported']['posts']))
			{
				$sites[$url]['imported']['posts'] = array();
			}

			if
			(
				!empty($base_post_type)
				&&
				isset($sites[$url]['available'][$base_post_type])
			)
			{
				$post_ids = (isset($sites[$url]['imported']['posts'][$base_post_type]))
				? array_diff($sites[$url]['available'][$base_post_type], array_keys($sites[$url]['imported']['posts'][$base_post_type]))
				: $sites[$url]['available'][$base_post_type];
				
				if (isset($sites[$url]['imported']['skipped']))
				{
					$post_ids = array_diff($post_ids, array_keys($sites[$url]['imported']['skipped']));
				}
				
				if (!empty($post_ids))
				{
					$args['post_id'] = array_shift($post_ids);
					$args[Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML] = $this->{Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML};
					$args[Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS] = $this->{Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS};
					$args[Directly_Import_Constants::HOOK_POST_META_IGNORE] = $this->{Directly_Import_Constants::HOOK_POST_META_IGNORE};
					$args[Directly_Import_Constants::HOOK_POST_META_POST_IDS] = $this->{Directly_Import_Constants::HOOK_POST_META_POST_IDS};
					$args[Directly_Import_Constants::HOOK_POST_META_TERM_IDS] = $this->{Directly_Import_Constants::HOOK_POST_META_TERM_IDS};
					$args[Directly_Import_Constants::HOOK_POST_META_USER_IDS] = $this->{Directly_Import_Constants::HOOK_POST_META_USER_IDS};
				}
			}
			
			$response = Directly_Import_Utilities::get_response(add_query_arg($args, $sites[$url]['api_url']));
			
			if ($response['success'])
			{
				$output['success'] = true;

				if (isset($response['objects']))
				{
					$output['continue'] = false;
					$output['imported'] = array();
					
					if (isset($response['objects']['users']))
					{
						if (!isset($sites[$url]['imported']['users']))
						{
							$sites[$url]['imported']['users'] = array();
						}

						foreach ($response['objects']['users'] as $old_user_id => $user_data)
						{
							if (!isset($sites[$url]['imported']['users'][$old_user_id]))
							{
								$existing_user = get_user_by('login', $response['objects']['users']['user_login']);

								if ($existing_user === false)
								{
									$existing_user = get_user_by('email', $response['objects']['users']['user_email']);
								}

								if ($existing_user === false)
								{
									$new_user_id = wp_insert_user($user_data);
									
									if (is_numeric($new_user_id))
									{
										$sites[$url]['imported']['users'][$old_user_id] = $new_user_id;
									}
								}
								else
								{
									$sites[$url]['imported']['users'][$old_user_id] = $existing_user->ID;
								}
							}
						}
					}

					if (isset($response['objects']['terms']))
					{
						if (!isset($sites[$url]['imported']['terms']))
						{
							$sites[$url]['imported']['terms'] = array();
						}

						foreach ($response['objects']['terms'] as $old_term_id => $term_data)
						{
							if (!isset($sites[$url]['imported']['terms'][$old_term_id]))
							{
								$args = array
								(
									'description' => $term_data['description'],
									'parent' => 0,
									'slug' => $term_data['slug'],
									
									'alias_of' => (isset($term_data['alias_of']))
									? $term_data['alias_of']
									: ''
								);
								
								if
								(
									$term_data['parent'] > 0
									&&
									isset($sites[$url]['imported']['terms'][$term_data['parent']])
								)
								{
									$args['parent'] = $sites[$url]['imported']['terms'][$term_data['parent']];
								}
								
								$new_term_id = wp_insert_term($term_data['name'], $term_data['taxonomy'], $args);
								
								if (is_array($new_term_id))
								{
									$sites[$url]['imported']['terms'][$old_term_id] = $new_term_id['term_id'];
								}
							}
						}
					}
					
					$post_parents = array();

					if (isset($response['objects']['posts']))
					{
						$current_user_id = get_current_user_id();
						$home_url = home_url('/');
						$all_acf_fields = $acf_field_names = array();

						$protocol_replacement = (strpos($sites[$url]['home_url'], 'http:') === false)
						? array('https:', 'http:')
						: array('http:', 'https:');

						$www_replacement = (strpos($sites[$url]['home_url'], '//www.') === false)
						? array('//', '//www.')
						: array('//www.', '//');
						
						$acf_fields_raw = get_posts(array
						(
							'post_type' => 'acf-field',
							'posts_per_page' => -1
						));

						foreach ($acf_fields_raw as $acf_field)
						{
							$all_acf_fields[$acf_field->ID] = $acf_field;
						}
						
						foreach ($all_acf_fields as $acf_field)
						{
							$meta_key_pieces = array();
							$processing = $acf_field;
							
							while (!empty($processing))
							{
								$meta_key_pieces[] = $acf_field->post_excerpt;
								
								if (isset($all_acf_fields[$processing->post_parent]))
								{
									$meta_key_pieces[] = '#';
									$processing = $all_acf_fields[$processing->post_parent];
								}
								else
								{
									$processing = null;
								}
							}
							
							$acf_field_names['_' . implode('_', array_reverse($meta_key_pieces))] = $acf_field->post_name;
						}
						
						foreach ($response['objects']['posts'] as $old_post_id => $post_data)
						{
							if
							(
								!isset($sites[$url]['imported']['posts'][$post_data['post_type']][$old_post_id])
								&&
								(
									$this->base->settings->any_post_type
									||
									$post_data['post_type'] == $base_post_type
									||
									$post_data['post_type'] == 'attachment'
									||
									!in_array($post_data['post_type'], array_keys($sites[$url]['available']))
									||
									
									/**
									 * Check to see if the current post should be imported based on its post type.
									 *
									 * @since 1.0.0
									 *
									 * @param  boolean $import            False by default.
									 * @param  string  $base_post_type    Post type being processed.
									 * @param  string  $current_post_type Post type for the current post.
									 * @return boolean                    True if the current post should be imported.
									 */
									apply_filters(Directly_Import_Constants::HOOK_IMPORT_POST_TYPE, false, $base_post_type, $post_data['post_type'])
								)
							)
							{
								if (!isset($sites[$url]['imported']['attachments']))
								{
									$sites[$url]['imported']['attachments'] = array();
								}

								$post_data['post_author'] = (isset($sites[$url]['imported']['users'][$post_data['post_author']]))
								? $sites[$url]['imported']['users'][$post_data['post_author']]
								: $current_user_id;
								
								$content_replacements = $sites[$url]['imported']['attachments'];
								
								if (isset($sites[$url]['imported']['posts']['attachment']))
								{
									foreach ($sites[$url]['imported']['posts']['attachment'] as $old_attachment_id => $new_attachment_id)
									{
										$content_replacements['wp-image-' . $old_attachment_id] = 'wp-image-' . $new_attachment_id;
									}
								}
								
								$content_replacements[$sites[$url]['home_url']] = $home_url;
								$content_replacements[str_replace($protocol_replacement[0], $protocol_replacement[1], $sites[$url]['home_url'])] = $home_url;
								$content_replacements[str_replace($www_replacement[0], $www_replacement[1], $sites[$url]['home_url'])] = $home_url;
								$content_replacements[str_replace($protocol_replacement[0] . $www_replacement[0], $protocol_replacement[1] . $www_replacement[1], $sites[$url]['home_url'])] = $home_url;
								
								$content_search = array_keys($content_replacements);
								$content_replace = array_values($content_replacements);
								
								$post_data['post_content'] = str_replace($content_search, $content_replace, $post_data['post_content']);
								
								if (!empty($post_data['post_category']))
								{
									foreach (array_keys($post_data['post_category']) as $old_category_id)
									{
										if (isset($sites[$url]['imported']['terms'][$old_category_id]))
										{
											$post_data['post_category'][$old_category_id] = $sites[$url]['imported']['terms'][$old_category_id];
										}
										else
										{
											unset($post_data['post_category'][$old_category_id]);
										}
									}
								}
								
								if (!empty($post_data['tags_input']))
								{
									foreach (array_keys($post_data['tags_input']) as $old_tag_id)
									{
										if (isset($sites[$url]['imported']['terms'][$old_tag_id]))
										{
											$post_data['tags_input'][$old_tag_id] = $sites[$url]['imported']['terms'][$old_tag_id];
										}
										else
										{
											unset($post_data['tags_input'][$old_tag_id]);
										}
									}
								}
								
								if (!empty($post_data['tax_input']))
								{
									foreach ($post_data['tax_input'] as $taxonomy => $term_ids)
									{
										if (!empty($term_ids))
										{
											foreach (array_keys($post_data['tax_input'][$taxonomy]) as $old_term_id)
											{
												if (isset($sites[$url]['imported']['terms'][$old_term_id]))
												{
													$post_data['tax_input'][$taxonomy][$old_term_id] = $sites[$url]['imported']['terms'][$old_term_id];
												}
												else
												{
													unset($post_data['tax_input'][$taxonomy][$old_term_id]);
												}
											}
										}
									}
								}
								
								if (!empty($post_data['meta_input']))
								{
									foreach ($post_data['meta_input'] as $meta_key => $meta_value)
									{
										$acf_meta_key = preg_replace('/_[0-9]+_/', '_#_', $meta_key);
										
										if (isset($acf_field_names[$acf_meta_key]))
										{
											$post_data['meta_input'][$meta_key] = $meta_value = $acf_field_names[$acf_meta_key];
										}
										
										if
										(
											in_array($meta_key, $this->{Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML})
											||
											in_array($meta_key, $this->{Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS})
										)
										{
											$post_data['meta_input'][$meta_key] = str_replace($content_search, $content_replace, $meta_value);
										}
										else if (is_numeric($meta_value))
										{
											if (in_array($meta_key, $this->{Directly_Import_Constants::HOOK_POST_META_POST_IDS}))
											{
												$post_found = false;
												
												foreach ($sites[$url]['imported']['posts'] as $post_type => $post_ids)
												{
													if (in_array($meta_value, array_keys($post_ids)))
													{
														$post_found = true;
														$post_data['meta_input'][$meta_key] = $post_ids[$meta_value];
														
														break;
													}
												}
												
												if (!$post_found)
												{
													unset($post_data['meta_input'][$meta_key]);
												}
											}
											else if (in_array($meta_key, $this->{Directly_Import_Constants::HOOK_POST_META_TERM_IDS}))
											{
												if (isset($sites[$url]['imported']['terms'][$meta_value]))
												{
													$post_data['meta_input'][$meta_key] = $sites[$url]['imported']['terms'][$meta_value];
												}
												else
												{
													unset($post_data['meta_input'][$meta_key]);
												}
											}
											else if (in_array($meta_key, $this->{Directly_Import_Constants::HOOK_POST_META_USER_IDS}))
											{
												if (isset($sites[$url]['imported']['users'][$meta_value]))
												{
													$post_data['meta_input'][$meta_key] = $sites[$url]['imported']['users'][$meta_value];
												}
												else
												{
													unset($post_data['meta_input'][$meta_key]);
												}
											}
										}
									}
								}
								
								$new_post_id = $message = '';
								
								if ($post_data['post_type'] == 'attachment')
								{
									if (isset($post_data['meta_input']['_wp_attached_file']))
									{
										$temp_file = download_url($sites[$url]['uploads_url'] . $post_data['meta_input']['_wp_attached_file']);
										
										if (is_wp_error($temp_file))
										{
											$message = $temp_file->get_error_message();
										}
										else
										{
											$pathinfo = pathinfo($post_data['meta_input']['_wp_attached_file']);
											$file_split = explode('/', $post_data['meta_input']['_wp_attached_file']);
											$upload_dir = 
											
											$file_details = array
											(
												'name' => $pathinfo['basename'],
												'tmp_name' => $temp_file
											);
											
											$file_date =
											(
												count($file_split) > 2
												&&
												is_numeric($file_split[0])
												&&
												strlen($file_split[0]) == 4
												&&
												is_numeric($file_split[1])
												&&
												strlen($file_split[1]) == 2
											)
											? $file_split[0] . '/' . $file_split[1]
											: date('Y/m', strtotime($post_data['post_date']));
											
											$file = wp_handle_sideload
											(
												$file_details,
												
												array
												(
													'test_form' => false
												),
												
												$file_date
											);
											
											if (isset($file['error']))
											{
												$message = $file['error'];
											}
											else
											{
												$post_data['guid'] = $file['url'];
												$post_data['post_mime_type'] = $file['type'];
												
												$new_post_id = wp_insert_attachment($post_data, $file['file'], $post_data['post_parent']);
												
												if (is_numeric($new_post_id))
												{
													$new_url = trailingslashit(wp_upload_dir($file_date)['url']) . $pathinfo['filename'];
													$sites[$url]['imported']['attachments'][$sites[$url]['uploads_url'] . $pathinfo['filename']] = $new_url;
													$sites[$url]['imported']['attachments'][str_replace($protocol_replacement[0], $protocol_replacement[1], $sites[$url]['uploads_url']) . $pathinfo['filename']] = $new_url;
													$sites[$url]['imported']['attachments'][str_replace($www_replacement[0], $www_replacement[1], $sites[$url]['uploads_url']) . $pathinfo['filename']] = $new_url;
													$sites[$url]['imported']['attachments'][str_replace($protocol_replacement[0] . $www_replacement[0], $protocol_replacement[1] . $www_replacement[1], $sites[$url]['uploads_url']) . $pathinfo['filename']] = $new_url;
													
													wp_update_attachment_metadata($new_post_id, wp_generate_attachment_metadata($new_post_id, $file['file']));
												}
											}
										}
										
										@unlink($temp_file);
									}
								}
								else
								{
									$new_post_id = wp_insert_post($post_data);
								}
								
								if (is_numeric($new_post_id))
								{
									if (!empty($post_data['post_parent']))
									{
										$post_parents[$new_post_id] = $post_data['post_parent'];
									}
									
									if (!isset($sites[$url]['imported']['posts'][$post_data['post_type']]))
									{
										$sites[$url]['imported']['posts'][$post_data['post_type']] = array();
									}

									$sites[$url]['imported']['posts'][$post_data['post_type']][$old_post_id] = $new_post_id;
									
									if (!isset($output['imported'][$post_data['post_type']]))
									{
										$output['imported'][$post_data['post_type']] = 0;
									}
									
									$output['imported'][$post_data['post_type']]++;
								}
								else
								{
									if (!isset($sites[$url]['imported']['skipped']))
									{
										$sites[$url]['imported']['skipped'] = array();
									}

									$sites[$url]['imported']['skipped'][$old_post_id] = $message;
								}
							}
						}
					}
					
					foreach ($post_parents as $post_id => $old_parent_id)
					{
						foreach ($sites[$url]['imported']['posts'] as $post_type => $post_ids)
						{
							if (in_array($old_parent_id, array_keys($post_ids)))
							{
								wp_update_post(array
								(
									'ID' => $post_id,
									'post_parent' => $post_ids[$old_parent_id]
								));
								
								break;
							}
						}
					}
					
					if (isset($sites[$url]['imported']['posts'][$base_post_type]))
					{
						$output['continue'] = (count(array_diff($sites[$url]['available'][$base_post_type], array_keys($sites[$url]['imported']['posts'][$base_post_type]))) > 0);
					}
				}
				else if (isset($response['available']))
				{
					$sites[$url]['available'] = $response['available'];
				}

				update_option(Directly_Import_Constants::OPTION_SITES, $sites);
			}
		}
		
		wp_send_json($output);
	}
	
	/**
	 * Regenerate the plugin API key.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function regenerate_api_key()
	{
		$this->base->settings->generate_api_key();
		
		wp_send_json(array
		(
			'api_key' => $this->base->settings->api_key,
			'success' => true,
			'url' => $this->base->settings->generate_url()
		));
	}
	
	/**
	 * Restore an import site.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function restore_site()
	{
		$url = (isset($_POST['url']))
		? esc_url_raw($_POST['url'])
		: '';
		
		$sites = Directly_Import_Utilities::check_array(get_option(Directly_Import_Constants::OPTION_SITES));
		
		if
		(
			!empty($url)
			&&
			isset($sites[$url])
		)
		{
			$sites[$url]['archived'] = false;
			
			update_option(Directly_Import_Constants::OPTION_SITES, $sites);
		}
		
		wp_send_json(array
		(
			'success' => true
		));
	}
}
