<?php
/*!
 * Base plugin functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Base
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the base plugin functionality.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import extends Directly_Import_Wrapper
{
	/**
	 * Main instance of Directly_Import.
	 *
	 * @since 1.0.0
	 *
	 * @access private static
	 * @var    Directly_Import
	 */
	private static $_instance = null;

	/**
	 * Returns the main instance of Directly_Import.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string          $file Main plugin file.
	 * @return Directly_Import       Main Directly_Import instance. 
	 */
	public static function _get_instance($file)
	{
		if (is_null(self::$_instance))
		{
			self::$_instance = new self($file);
		}

		return self::$_instance;
	}

	/**
	 * File path for the plugin.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var    string
	 */
	public $plugin;

	/**
	 * Global cache object.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var    Directly_Import_Cache
	 */
	public $cache;

	/**
	 * Global settings object.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var    Directly_Import_Settings
	 */
	public $settings;

	/**
	 * Global tools object.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var    Directly_Import_Tools
	 */
	public $tools;

	/**
	 * Global export object.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var    Directly_Import_Export
	 */
	public $export;

	/**
	 * Global AJAX object.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var    Directly_Import_Ajax
	 */
	public $ajax;

	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  string $file Main plugin file.
	 * @return void
	 */
	public function __construct($file)
	{
		if
		(
			!empty($file)
			&&
			file_exists($file)
		)
		{
			$this->plugin = $file;

			add_action('plugins_loaded', array($this, 'plugins_loaded'));
		}
	}

	/**
	 * Load the plugin functionality.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function plugins_loaded()
	{
		$this->cache = new Directly_Import_Cache();
		$this->settings = new Directly_Import_Settings();
		
		if (!$this->settings->disable_imports)
		{
			$this->tools = new Directly_Import_Tools();
		}
		
		if (!$this->settings->disable_exports)
		{
			$this->export = new Directly_Import_Export();
		}
		
		if
		(
			defined('DOING_AJAX')
			&&
			DOING_AJAX
		)
		{
			$this->ajax = new Directly_Import_Ajax();
		}

		add_action('admin_init', array('Directly_Import_Setup', 'check_version'), 0);
		add_action('init', array($this, 'init'));
		
		add_filter('plugin_row_meta', array($this, 'plugin_row_meta'), 10, 2);
	}

	/**
	 * Initialize the plugin.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function init()
	{
		load_plugin_textdomain('directly-import', false, dirname(plugin_basename($this->plugin)) . '/languages/');
	}

	/**
	 * Add links to the plugin page.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  array  $links Default links for the plugin.
	 * @param  string $file  Main plugin file name.
	 * @return array         Modified links for the plugin.
	 */
	public function plugin_row_meta($links, $file)
	{
		if ($file == plugin_basename($this->plugin))
		{
			$links[] = '<a href="' . Directly_Import_Constants::URL_SUPPORT . '" target="_blank" rel="noopener noreferrer">' . __('Support', 'directly-import') . '</a>';
			$links[] = '<a href="' . Directly_Import_Constants::URL_REVIEW . '" target="_blank" rel="noopener noreferrer">' . __('Review', 'directly-import') . '</a>';
			$links[] = '<a href="' . Directly_Import_Constants::URL_TRANSLATE . '" target="_blank" rel="noopener noreferrer">' . __('Translate', 'directly-import') . '</a>';
			$links[] = '<a href="' . Directly_Import_Constants::URL_DONATE . '" target="_blank" rel="noopener noreferrer">' . __('Donate', 'directly-import') . '</a>';
		}

		return $links;
	}
}
