<?php
/*!
 * Cached function calls and flags.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Cache
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the cache functionality.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import_Cache extends Directly_Import_Wrapper
{
	/**
	 * Get a default cached item based on the provided name.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 * @param  string $name Name of the cached item to return.
	 * @return mixed        Default cached item if it exists, otherwise an empty string.
	 */
	protected function _default($name)
	{
		switch ($name)
		{
			/**
			 * Path to the plugin assets folder.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'assets_url':

				$folder = 'debug';
				
				if
				(
					!$this->script_debug
					||
					!file_exists(plugin_dir_path($this->base->plugin) . 'assets/' . $folder . '/')
				)
				{
					$folder = 'release';
				}

				return plugins_url('/assets/' . $folder . '/', $this->base->plugin);

			/**
			 * Current page being viewed.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'current_page':

				return
				(
					isset($_GET['page'])
					&&
					!empty($_GET['page'])
				)
				? sanitize_key($_GET['page'])
				: '';

			/**
			 * Validation rules for the current form.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'form_validation':

				return array();

			/**
			 * Asset file names pulled from the manifest JSON.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'manifest':

				return Directly_Import_Utilities::load_json('assets/manifest.json');

			/**
			 * General details about the plugin.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'plugin_data':

				return (function_exists('get_plugin_data'))
				? get_plugin_data($this->base->plugin)
				: array
				(
					'AuthorName' => 'Robert Noakes',
					'Name' => 'Directly Import'
				);

			/**
			 * Object for the current screen.
			 *
			 * @since 1.0.0
			 *
			 * @var WP_Screen
			 */
			case 'screen':

				return (function_exists('get_current_screen'))
				? get_current_screen()
				: new WP_Screen();
				
			/**
			 * True if script debugging is enabled.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'script_debug':
			
				return
				(
					defined('SCRIPT_DEBUG')
					&&
					SCRIPT_DEBUG
				);
		}

		return parent::_default($name);
	}

	/**
	 * Obtain a path to an asset.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  string $path      Path to the asset folder.
	 * @param  string $file_name File name for the asset.
	 * @return string            Full path to the requested asset.
	 */
	public function asset_path($path, $file_name)
	{
		$manifest = $this->manifest;

		if (isset($manifest[$file_name]))
		{
			$file_name = $manifest[$file_name];
		}

		return trailingslashit($this->assets_url . $path) . $file_name;
	}
}

