<?php
/*!
 * Export functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Export
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the export functionality.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import_Export extends Directly_Import_Wrapper
{
	/**
	 * Added object IDs.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private $_added = array
	(
		'comments' => array(),
		'posts' => array(),
		'terms' => array(),
		'users' => array()
	);
	
	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
		
		add_action('init', array($this, 'init'));
		
		add_filter('query_vars', array($this, 'query_vars'));
		add_filter('template_redirect', array($this, 'template_redirect'));
	}
	
	/**
	 * Get a default value based on the provided name.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 * @param  string $name Name of the value to return.
	 * @return mixed        Default value if it exists, otherwise an empty string.
	 */
	protected function _default($name)
	{
		switch ($name)
		{
			/**
			 * Current import action.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'import_action':
			
				return (isset($_GET['import_action']))
				? sanitize_key($_GET['import_action'])
				: '';
				
			/**
			 * Current post ID.
			 *
			 * @since 1.0.0
			 *
			 * @var integer
			 */
			case 'post_id':
			
				return
				(
					isset($_GET['post_id'])
					&&
					is_numeric($_GET['post_id'])
				)
				? sanitize_text_field($_GET['post_id'])
				: 0;
				
			/**
			 * User meta keys to include in the export.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'user_meta_keys':
			
				return array('nickname', 'first_name', 'last_name', 'description', 'rich_editing', 'syntax_highlighting', 'comment_shortcuts', 'admin_color', 'use_ssl', 'show_admin_bar_front', 'locale');
				
			/**
			 * Post meta keys containing HTML content.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML:
			
				return
				(
					isset($_GET[Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML])
					&&
					is_array($_GET[Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML])
				)
				? array_map('sanitize_key', $_GET[Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML])
				: array();
				
			/**
			 * Post meta keys containing post URLs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS:
			
				return
				(
					isset($_GET[Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS])
					&&
					is_array($_GET[Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS])
				)
				? array_map('sanitize_key', $_GET[Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS])
				: array();
				
			/**
			 * Post meta keys and prefixes that should be ignored.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_IGNORE:
			
				return
				(
					isset($_GET[Directly_Import_Constants::HOOK_POST_META_IGNORE])
					&&
					is_array($_GET[Directly_Import_Constants::HOOK_POST_META_IGNORE])
				)
				? array_map('sanitize_key', $_GET[Directly_Import_Constants::HOOK_POST_META_IGNORE])
				: array();
				
			/**
			 * Post meta keys containing post IDs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_POST_IDS:
			
				return
				(
					isset($_GET[Directly_Import_Constants::HOOK_POST_META_POST_IDS])
					&&
					is_array($_GET[Directly_Import_Constants::HOOK_POST_META_POST_IDS])
				)
				? array_map('sanitize_key', $_GET[Directly_Import_Constants::HOOK_POST_META_POST_IDS])
				: array();
				
			/**
			 * Post meta keys containing term IDs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_TERM_IDS:
			
				return
				(
					isset($_GET[Directly_Import_Constants::HOOK_POST_META_TERM_IDS])
					&&
					is_array($_GET[Directly_Import_Constants::HOOK_POST_META_TERM_IDS])
				)
				? array_map('sanitize_key', $_GET[Directly_Import_Constants::HOOK_POST_META_TERM_IDS])
				: array();
				
			/**
			 * Post meta keys containing user IDs.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case Directly_Import_Constants::HOOK_POST_META_USER_IDS:
			
				return
				(
					isset($_GET[Directly_Import_Constants::HOOK_POST_META_USER_IDS])
					&&
					is_array($_GET[Directly_Import_Constants::HOOK_POST_META_USER_IDS])
				)
				? array_map('sanitize_key', $_GET[Directly_Import_Constants::HOOK_POST_META_USER_IDS])
				: array();
		}

		return parent::_default($name);
	}

	/**
	 * Initialize the rewrite rule.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function init()
	{
		add_rewrite_rule('^' . Directly_Import_Constants::TOKEN . '/([^/]*)/?', 'index.php?' . Directly_Import_Constants::TOKEN . '=$matches[1]', 'top');
	}
	
	/**
	 * Add the plugin query var.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  array $vars Current query vars for the plugin.
	 * @return array       Modified query vars for the plugin.
	 */
	public function query_vars($vars)
	{
		$vars[] = Directly_Import_Constants::TOKEN;
		
		return $vars;
	}
	
	/**
	 * Output the appropriate export JSON.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function template_redirect()
	{
		if
		(
			is_404()
			&&
			strpos($_SERVER['REQUEST_URI'], '?' . Directly_Import_Constants::TOKEN . '=') !== false
			&&
			!isset($_GET['diim-flush'])
		)
		{
			flush_rewrite_rules(false);
			wp_redirect(add_query_arg('diim-flush', true));
			
			exit;
		}
		
		$query_var = get_query_var(Directly_Import_Constants::TOKEN);
		
		if (!empty($query_var))
		{
			$output = array
			(
				'success' => false
			);

			if ($query_var === $this->base->settings->api_key)
			{
				if
				(
					!empty($this->import_action)
					&&
					method_exists($this, '_action_' . $this->import_action)
				)
				{
					call_user_func(array($this, '_action_' . $this->import_action));
				}
				else
				{
					$output['home_url'] = home_url('/');
					$output['uploads_url'] = trailingslashit(wp_upload_dir()['baseurl']);
				}

				$output['success'] = true;
			}
			
			wp_send_json($output);
		}
	}
	
	/**
	 * Import a complete post.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return void
	 */
	private function _action_import()
	{
		$output = array
		(
			'success' => true
		);
		
		if (empty($this->post_id))
		{
			$output['success'] = false;
		}
		else
		{
			set_time_limit(0);
			
			$output['objects'] = $this->_add_post($this->post_id);
		}
		
		wp_send_json($output);
	}
	
	/**
	 * Add a post for the export.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  integer $post_id ID for the post to retrieve.
	 * @param  array   $objects Objects processed so far.
	 * @return array            Updated objects for the current export.
	 */
	private function _add_post($post_id, $objects = array())
	{
		if (!is_numeric($post_id))
		{
			return $objects;
		}
		
		if (!isset($objects['posts']))
		{
			$objects['posts'] = array();
		}
		
		$current_post = get_post($post_id);
		
		if
		(
			!empty($current_post)
			&&
			!in_array($post_id, $this->_added['posts'])
		)
		{
			$this->_added['posts'][] = $post_id;
			
			if (!empty($current_post->post_parent))
			{
				$objects = $this->_add_post($current_post->post_parent, $objects);
			}
			
			$objects = $this->_process_html($current_post->post_content, $objects);
			$post_meta = $this->_prepare_meta(get_post_meta($post_id), $this->{Directly_Import_Constants::HOOK_POST_META_IGNORE});
			$post_meta_keys = array_keys($post_meta);
			$post_meta_contain_html = array_intersect($this->{Directly_Import_Constants::HOOK_POST_META_CONTAIN_HTML}, $post_meta_keys);
			$post_meta_contain_urls = array_intersect($this->{Directly_Import_Constants::HOOK_POST_META_CONTAIN_URLS}, $post_meta_keys);
			$post_meta_post_ids = array_intersect($this->{Directly_Import_Constants::HOOK_POST_META_POST_IDS}, $post_meta_keys);
			$post_meta_term_ids = array_intersect($this->{Directly_Import_Constants::HOOK_POST_META_TERM_IDS}, $post_meta_keys);
			$post_meta_user_ids = array_intersect($this->{Directly_Import_Constants::HOOK_POST_META_USER_IDS}, $post_meta_keys);
			
			foreach ($post_meta_contain_html as $meta_key)
			{
				$objects = $this->_process_html($post_meta[$meta_key], $objects);
			}
			
			foreach ($post_meta_contain_urls as $meta_key)
			{
				$objects = $this->_process_url($post_meta[$meta_key], $objects);
			}
			
			foreach ($post_meta_post_ids as $meta_key)
			{
				if (is_numeric($post_meta[$meta_key]))
				{
					$objects = $this->_add_post($post_meta[$meta_key], $objects);
				}
			}
			
			foreach ($post_meta_term_ids as $meta_key)
			{
				if (is_numeric($post_meta[$meta_key]))
				{
					$objects = $this->_add_term($post_meta[$meta_key], $objects);
				}
			}
			
			foreach ($post_meta_user_ids as $meta_key)
			{
				if (is_numeric($post_meta[$meta_key]))
				{
					$objects = $this->_add_user($post_meta[$meta_key], $objects);
				}
			}

			$objects['posts'][$current_post->ID] = $current_post->to_array();
			$objects['posts'][$current_post->ID]['meta_input'] = $post_meta;
			$objects['posts'][$current_post->ID]['post_category'] = $objects['posts'][$current_post->ID]['tags_input'] = $objects['posts'][$current_post->ID]['tax_input'] = array();
			
			unset($objects['posts'][$current_post->ID]['ID']);
			unset($objects['posts'][$current_post->ID]['guid']);

			$terms = wp_get_post_terms($post_id, get_object_taxonomies($current_post->post_type));
			
			if (!empty($terms))
			{
				foreach ($terms as $term)
				{
					if (!isset($objects['terms'][$term->term_id]))
					{
						if ($term->taxonomy == 'category')
						{
							$objects['posts'][$current_post->ID]['post_category'][$term->term_id] = $term->term_id;
						}
						else if ($term->taxonomy == 'post_tag')
						{
							$objects['posts'][$current_post->ID]['tags_input'][$term->term_id] = $term->term_id;
						}
						else
						{
							if (!isset($objects['posts'][$current_post->ID]['tax_input'][$term->taxonomy]))
							{
								$objects['posts'][$current_post->ID]['tax_input'][$term->taxonomy] = array();
							}
							
							$objects['posts'][$current_post->ID]['tax_input'][$term->taxonomy][$term->term_id] = $term->term_id;
						}
						
						$objects = $this->_add_term($term, $objects);
					}
				}
			}

			$objects = $this->_add_user($current_post->post_author, $objects);
		}
		
		return $objects;
	}
	
	/**
	 * Prepare meta to be exported.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  array $all_meta The meta data that should be filtered.
	 * @param  array $ignore   Meta key and prefixes that should be ignored.
	 * @return array           Filter meta data.
	 */
	private function _prepare_meta($all_meta, $ignore)
	{
		global $wpdb;
		
		$output = array();

		foreach ($all_meta as $meta_key => $meta_value)
		{
			if (!in_array($meta_key, $ignore))
			{
				$add_meta = true;

				foreach ($ignore as $ignore_key)
				{
					if (Directly_Import_Utilities::starts_with($ignore_key, $meta_key))
					{
						$add_meta = false;

						break;
					}
				}

				if ($add_meta)
				{
					$output[$meta_key] = $meta_value[0];
				}
			}
		}
		
		return $output;
	}
	
	/**
	 * Process attachments that appear in HTML content.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  string $content Raw HTML content to process.
	 * @param  array  $objects Objects processed so far.
	 * @return array           Updated objects for the current export.
	 */
	private function _process_html($content, $objects)
	{
		$dom = new DOMDocument();
		$dom->loadHTML(wpautop(do_shortcode($content)));

		$uploads_url = trailingslashit(wp_upload_dir()['baseurl']);

		foreach ($dom->getElementsByTagName('a') as $a)
		{
			$href = $a->getAttribute('href');

			if (Directly_Import_Utilities::starts_with($uploads_url, $href))
			{
				$objects = $this->_process_url($href, $objects);
			}
		}

		foreach ($dom->getElementsByTagName('img') as $img)
		{
			$classes = explode(' ', $img->getAttribute('class'));

			foreach ($classes as $class)
			{
				if (Directly_Import_Utilities::starts_with('wp-image-', $class))
				{
					$objects = $this->_add_post(str_replace('wp-image-', '', $class), $objects);
				}
			}
		}
		
		return $objects;
	}
	
	/**
	 * Process a post URL.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  string $url     Post URL to process.
	 * @param  array  $objects Objects processed so far.
	 * @return array           Updated objects for the current export.
	 */
	private function _process_url($url, $objects)
	{
		$replacements = (strpos($url, 'http:') === false)
		? array('https:' => 'http:')
		: array('http:' => 'https:');
		
		$objects = $this->_add_post(attachment_url_to_postid($url), $objects);
		$objects = $this->_add_post(attachment_url_to_postid(str_replace(array_keys($replacements), array_values($replacements), $url)), $objects);
		
		return $objects;
	}
	
	/**
	 * Add a term for the export.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  WP_Term/integer $term_or_id Object or ID for the term being processed.
	 * @param  array           $objects    Objects processed so far.
	 * @return array                       Updated objects for the current export.
	 */
	private function _add_term($term_or_id, $objects)
	{
		$term = (is_numeric($term_or_id))
		? get_term($term_or_id)
		: $term_or_id;
		
		if (!is_a($term, 'WP_Term'))
		{
			return $objects;
		}
		
		if (!isset($objects['terms']))
		{
			$objects['terms'] = array();
		}
		
		if
		(
			!empty($term->parent)
			&&
			!isset($objects['terms'][$term->parent])
		)
		{
			$objects = $this->_add_term($term->parent, $objects);
		}
		
		if (!in_array($term->term_id, $this->_added['terms']))
		{
			$this->_added['terms'][] = $term->term_id;
			
			$objects['terms'][$term->term_id] = $term->to_array();
			
			unset($objects['terms'][$term->term_id]['term_id']);
			unset($objects['terms'][$term->term_id]['count']);
		}
		
		return $objects;
	}
	
	/**
	 * Add a user for the export.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  WP_User/integer $user_or_id Object or ID for the user being processed.
	 * @param  array           $objects    Objects processed so far.
	 * @return array                       Updated objects for the current export.
	 */
	private function _add_user($user_or_id, $objects)
	{
		$user = (is_numeric($user_or_id))
		? get_user_by('id', $user_or_id)
		: $user_or_id;
		
		if (!is_a($user, 'WP_User'))
		{
			return $objects;
		}
		
		if (!isset($objects['users']))
		{
			$objects['users'] = array();
		}
		
		if (!in_array($user->ID, $this->_added['users']))
		{
			$this->_added['users'][] = $user->ID;
			
			$user_data = array
			(
				'user_login' => $user->data->user_login,
				'user_nicename' => $user->data->user_nicename,
				'user_email' => $user->data->user_email,
				'user_url' => $user->data->user_url,
				'user_registered' => $user->data->user_registered,
				'display_name' => $user->data->display_name
			);
			
			$all_user_meta = get_user_meta($user->ID);
			
			foreach ($this->user_meta_keys as $meta_key)
			{
				if (isset($all_user_meta[$meta_key]))
				{
					$user_data[$meta_key] = $all_user_meta[$meta_key][0];
				}
			}
			
			$objects['users'][$user->ID] = $user_data;
		}
		
		return $objects;
	}
	
	/**
	 * Get an import summary.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return void
	 */
	private function _action_summary()
	{
		$output = array
		(
			'success' => true
		);
		
		$args = array
		(
			'order' => 'ASC',
			'post_status' => array('publish', 'future', 'draft', 'pending', 'private', 'inherit'),
			'post_type' => 'any',
			'posts_per_page' => -1
		);
		
		$all_posts = get_posts($args);
		
		if (!empty($all_posts))
		{
			$output['available'] = array();
			
			foreach ($all_posts as $current_post)
			{
				if (!isset($output['available'][$current_post->post_type]))
				{
					$output['available'][$current_post->post_type] = array();
				}
				
				$output['available'][$current_post->post_type][] = $current_post->ID;
			}
		}
		
		$attachment_post_type = $args['post_type'] = 'attachment';
		$attachments = get_posts($args);
		
		if (!empty($attachments))
		{
			if (!isset($output['available']))
			{
				$output['available'] = array();
			}
			
			$output['available'][$attachment_post_type] = array();
			
			foreach ($attachments as $current_post)
			{
				$output['available'][$attachment_post_type][] = $current_post->ID;
			}
		}
		
		wp_send_json($output);
	}
}
