<?php
/*!
 * Settings functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Settings
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the settings functionality.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import_Settings extends Directly_Import_Wrapper
{
	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->load_option();
		
		add_action('admin_init', array($this, 'admin_init'));
		add_action('admin_menu', array($this, 'admin_menu'));
		
		add_filter('plugin_action_links_' . plugin_basename($this->base->plugin), array($this, 'plugin_action_links'));
	}
	
	/**
	 * Get a default value based on the provided name.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 * @param  string $name Name of the value to return.
	 * @return mixed        Default value if it exists, otherwise an empty string.
	 */
	protected function _default($name)
	{
		switch ($name)
		{
			/**
			 * API key for the export URL.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'api_key':
			
				return '';
				
			/**
			 * True if imports should be disabled on this site.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'disable_imports':
				
			/**
			 * True if any post type should be processed.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'any_post_type':
			
				return false;
				
			/**
			 * True if exports should be disabled on this site.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'disable_exports':
			
				return true;
				
			/**
			 * True if plugin settings should be deleted.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case Directly_Import_Constants::SETTING_DELETE_SETTINGS:
			
			/**
			 * True if plugin settings should be deleted.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case Directly_Import_Constants::SETTING_DELETE_SETTINGS . Directly_Import_Constants::SETTING_UNCONFIRMED:
			
				return false;
		}

		return parent::_default($name);
	}

	/**
	 * Load the settings option.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  array $settings Setting array to load, or null of the settings should be loaded from the database.
	 * @return void
	 */
	public function load_option($settings = null)
	{
		if (empty($settings))
		{
			$settings = get_option(Directly_Import_Constants::OPTION_SETTINGS);
		}
		
		$this->_set_properties($settings);
		
		if (empty($this->api_key))
		{
			$this->generate_api_key();
		}
	}

	/**
	 * Register the settings option.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function admin_init()
	{
		register_setting
		(
			Directly_Import_Constants::OPTION_SETTINGS,
			Directly_Import_Constants::OPTION_SETTINGS,
			
			array
			(
				'sanitize_callback' => array($this, 'sanitize_callback')
			)
		);
	}

	/**
	 * Sanitize callback for the settings.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  array $input Raw settings array.
	 * @return array        Sanitized settings array.
	 */
	public function sanitize_callback($input)
	{
		if
		(
			!is_array($input)
			||
			empty($input)
		)
		{
			return array();
		}
		
		$booleans = array('disable_imports', 'disable_exports', 'all_post_types');
		
		foreach ($booleans as $boolean)
		{
			$input[$boolean] =
			(
				isset($input[$boolean])
				&&
				!empty($input[$boolean])
			);
		}
		
		foreach ($input as $name => $value)
		{
			if ($name == Directly_Import_Constants::SETTING_DELETE_SETTINGS)
			{
				$delete_settings_unconfirmed = Directly_Import_Constants::SETTING_DELETE_SETTINGS . Directly_Import_Constants::SETTING_UNCONFIRMED;

				$input[$name] =
				(
					isset($input[$delete_settings_unconfirmed])
					&&
					$input[$delete_settings_unconfirmed]
				)
				? $value
				: false;
			}
			else
			{
				$input[$name] = sanitize_text_field($value);
			}
		}

		return $input;
	}

	/**
	 * Add the settings menu item.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function admin_menu()
	{
		$this->_page_title = __('Settings', 'directly-import');

		$settings_page = add_options_page($this->_page_title, $this->base->cache->plugin_data['Name'], 'manage_options', Directly_Import_Constants::OPTION_SETTINGS, array($this, 'settings_page'));

		if ($settings_page)
		{
			add_action('load-' . $settings_page, array($this, 'load_settings_page'));
		}
	}

	/**
	 * Output the settings page.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function settings_page()
	{
		Directly_Import_Global::admin_tabs();
		Directly_Import_Output::admin_form_page($this->_page_title, Directly_Import_Constants::OPTION_SETTINGS);
	}

	/**
	 * Load settings page functionality.
	 * 
	 * @since 1.0.0
	 * 
	 * @access public
	 * @return void
	 */
	public function load_settings_page()
	{
		add_action('admin_enqueue_scripts', array('Directly_Import_Global', 'admin_enqueue_scripts_form'), 1000);

		add_filter('admin_body_class', array('Directly_Import_Global', 'admin_body_class'));
		
		add_screen_option
		(
			'layout_columns',

			array
			(
				'default' => 2,
				'max' => 2
			)
		);

		$this->_add_meta_boxes();

		Directly_Import_Help::output('settings');
	}

	/**
	 * Add meta boxes to the settings page.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return void
	 */
	private function _add_meta_boxes()
	{
		$import_settings_box = new Directly_Import_Meta_Box(array
		(
			'context' => 'normal',
			'id' => 'import_settings',
			'option_name' => Directly_Import_Constants::OPTION_SETTINGS,
			'title' => __('Import Settings', 'directly-import')
		));
		
		$import_settings_box->add_field(array
		(
			'description' => __('If checked, imports will be disabled on this site.', 'directly-import'),
			'label' => __('Disable Imports', 'directly-import'),
			'name' => 'disable_imports',
			'type' => 'checkbox',
			'value' => $this->disable_imports
		));
		
		$import_settings_box->add_field(array
		(
			'description' => __('If checked, any post type will be processed. Otherwise, only the selected post type, unregistered post types and attachments will be processed.', 'directly-import'),
			'label' => __('Any Post Type', 'directly-import'),
			'name' => 'any_post_type',
			'type' => 'checkbox',
			'value' => $this->any_post_type
		));
		
		$save_all_settings = array
		(
			'content' => __('Save All Settings', 'directly-import'),
			'type' => 'submit'
		);
		
		$import_settings_box->add_field($save_all_settings);
		
		$export_settings_box = new Directly_Import_Meta_Box(array
		(
			'context' => 'normal',
			'id' => 'export_settings',
			'option_name' => Directly_Import_Constants::OPTION_SETTINGS,
			'title' => __('Export Settings', 'directly-import')
		));
		
		$export_settings_box->add_field(array
		(
			'classes' => array('diim-hidden'),
			'input_classes' => array('diim-api-key'),
			'name' => 'api_key',
			'type' => 'hidden',
			'value' => $this->api_key
		));
		
		$export_settings_box->add_field(array
		(
			'description' => __('If checked, exports will be unavailable from this site. It is recommened that you disable exports when the plugin is not being used.', 'directly-import'),
			'label' => __('Disable Exports', 'directly-import'),
			'name' => 'disable_exports',
			'type' => 'checkbox',
			'value' => $this->disable_exports
		));
		
		$export_settings_box->add_field(array
		(
			'classes' => array('diim-export-url'),
			'description' => __('Implement this URL into the site you will import from.', 'directly-import'),
			'is_tall' => true,
			'label' => __('Export URL', 'directly-import'),
			'type' => 'html',
			
			'content' => '<pre class="diim-export-url">' . $this->generate_url() . '</pre>'
			. '<button type="button" class="button diim-button diim-copy-to-clipboard">' . __('Copy to Clipboard', 'directly-import') . '</button> '
			. '<button type="button" class="button diim-button diim-ajax-button diim-regenerate-api-key">' . __('Regenerate API Key', 'directly-import') . '</button>'
		));
		
		$export_settings_box->add_field($save_all_settings);
		
		$uninstall_settings_box = new Directly_Import_Meta_Box(array
		(
			'context' => 'normal',
			'id' => 'uninstall_settings',
			'option_name' => Directly_Import_Constants::OPTION_SETTINGS,
			'title' => __('Uninstall Settings', 'directly-import')
		));

		$delete_settings_description = sprintf
		(
			__('Delete settings for %1$s when the plugin is uninstalled.', 'directly-import'),
			$plugin_label
		);

		$delete_settings_label = __('Delete Plugin Settings', 'directly-import');
		$delete_settings_unconfirmed = Directly_Import_Constants::SETTING_DELETE_SETTINGS . Directly_Import_Constants::SETTING_UNCONFIRMED;
		$delete_settings_value = $this->{Directly_Import_Constants::SETTING_DELETE_SETTINGS};

		$uninstall_settings_box->add_field(array
		(
			'description' => $delete_settings_description,
			'label' => $delete_settings_label,
			'name' => $delete_settings_unconfirmed,
			'type' => 'checkbox',
			'value' => $delete_settings_value,

			'classes' => ($delete_settings_value)
			? array('diim-hidden')
			: array()
		));

		$uninstall_settings_box->add_field(array
		(
			'description' => $delete_settings_description,
			'name' => Directly_Import_Constants::SETTING_DELETE_SETTINGS,
			'type' => 'checkbox',
			'value' => $delete_settings_value,

			'classes' => ($delete_settings_value)
			? array()
			: array('diim-confirmation diim-hidden'),

			'conditional' => array
			(
				array
				(
					'field' => $delete_settings_unconfirmed,
					'value' => '1'
				)
			),

			'label' => ($delete_settings_value)
			? $delete_settings_label
			: __('Confirm Delete Plugin Settings', 'directly-import')
		));

		$uninstall_settings_box->add_field($save_all_settings);

		Directly_Import_Meta_Box::side_meta_boxes();
		Directly_Import_Meta_Box::finalize_meta_boxes();
	}

	/**
	 * Add settings to the plugin action links.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  array $links Existing action links.
	 * @return array        Modified action links.
	 */
	public function plugin_action_links($links)
	{
		array_unshift($links, '<a href="' . esc_url(get_admin_url(null, 'options-general.php?page=' . Directly_Import_Constants::OPTION_SETTINGS)) . '">' . __('Settings', 'directly-import') . '</a>');

		return $links;
	}
	
	/**
	 * Generate an API key.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function generate_api_key()
	{
		$this->api_key = md5(home_url() . '|' . time() . '|' . rand());
		
		update_option(Directly_Import_Constants::OPTION_SETTINGS, $this->_properties);
	}
	
	/**
	 * Generate the export URL.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return string Generated export URL.
	 */
	public function generate_url()
	{
		return esc_url(home_url('index.php?' . Directly_Import_Constants::TOKEN . '=' . $this->api_key));
	}
}
