<?php
/*!
 * Tools functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Tools
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the tools functionality.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import_Tools extends Directly_Import_Wrapper
{
	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
		
		add_action('admin_menu', array($this, 'admin_menu'));
	}

	/**
	 * Add the tools menu item.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function admin_menu()
	{
		$this->_page_title = __('Sites', 'directly-import');

		$tools_page = add_submenu_page('tools.php', $this->_page_title, $this->base->cache->plugin_data['Name'], 'manage_options', Directly_Import_Constants::OPTION_SITES, array($this, 'tools_page'));

		if ($tools_page)
		{
			add_action('load-' . $tools_page, array($this, 'load_tools_page'));
		}
	}

	/**
	 * Output the tools page.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function tools_page()
	{
		Directly_Import_Global::admin_tabs();
		Directly_Import_Output::admin_form_page($this->_page_title);
	}

	/**
	 * Load tools page functionality.
	 * 
	 * @since 1.0.0
	 * 
	 * @access public
	 * @return void
	 */
	public function load_tools_page()
	{
		add_action('admin_enqueue_scripts', array('Directly_Import_Global', 'admin_enqueue_scripts_form'), 1000);

		add_filter('admin_body_class', array('Directly_Import_Global', 'admin_body_class'));
		
		add_screen_option
		(
			'layout_columns',

			array
			(
				'default' => 2,
				'max' => 2
			)
		);

		$this->_add_meta_boxes();

		Directly_Import_Help::output('incoming');
	}

	/**
	 * Add meta boxes to the tools page.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @return void
	 */
	public function _add_meta_boxes()
	{
		$sites = Directly_Import_Utilities::check_array(get_option(Directly_Import_Constants::OPTION_SITES));
		$archived = array();
		
		foreach ($sites as $url => $details)
		{
			if ($details['archived'])
			{
				$archived[] = $url;
			}
			else
			{
				$site_box = new Directly_Import_Meta_Box(array
				(
					'context' => 'normal',
					'id' => sanitize_key($url),
					'title' => $url
				));

				$site_box->add_field(array
				(
					'classes' => array('diim-hidden'),
					'content' => '<input type="hidden" value="' . esc_attr($url) . '" class="diim-site-url" />',
					'type' => 'html'
				));
				
				$summary = '';
				$import_fields = array();
				
				if (empty($details['available']))
				{
					$summary = '<p class="diim-text-large">' . __('Click on the button below to pull an import summary for this site.', 'directly-import') . '</p>';
				}
				else
				{
					$summary = '<p class="diim-text-large"><strong>' . __('Available to Import:', 'directly-import') . '</strong></p>'
					. '<ul>';
					
					$total_label = _x('%1$s - Total: %2$d', 'Post Type, Total', 'directly-import');
					$import_label = __('Import Remaining', 'directly-import');
					$post_type_label = __('Post Type: %1$s', 'directly-import');
					$imported_label = __('%1$s of %2$d Imported', 'directly-import');
					
					foreach ($details['available'] as $post_type => $posts)
					{
						$post_count = count($posts);
						$summary .= '<li>' . sprintf($total_label, '<strong>' . $post_type . '</strong>', $post_count) . '</li>';
						
						$posts_imported = (isset($details['imported']['posts'][$post_type]))
						? count(array_intersect($details['available'][$post_type], array_keys($details['imported']['posts'][$post_type])))
						: 0;
						
						$post_type = esc_attr($post_type);
						
						$import_button = ($posts_imported < $post_count)
						? '<p><button type="button" class="button diim-button diim-ajax-button diim-import-button" data-diim-import-action="import" data-diim-post-type="' . $post_type . '">' . $import_label . '</button>'
						: '';
						
						$import_fields[] = array
						(
							'label' => sprintf($post_type_label, $post_type),
							'type' => 'html',
							
							'content' => '<p>' . sprintf($imported_label, '<span class="diim-imported diim-post-type-' . $post_type . '">' . $posts_imported . '</span>', $post_count) . '</p>'
							. $import_button
						);
					}
					
					$summary .= '</ul>';
				}
				
				$site_box->add_field(array
				(
					'description' => __('Summary of the information available on the export site.', 'directly-import'),
					'is_tall' => true,
					'label' => __('Summary', 'directly-import'),
					'type' => 'html',
					
					'content' => $summary
					. '<p><button type="button" class="button diim-button diim-ajax-button" data-diim-import-action="summary">' . __('Check Posts', 'directly-import') . '</button></p>'
				));
				
				foreach ($import_fields as $import_field)
				{
					$site_box->add_field($import_field);
				}

				$site_box->add_field(array
				(
					'content' => '<button type="button" class="button diim-button diim-ajax-button diim-archive-site">' . __('Archive Site', 'directly-import') . '</button>',
					'type' => 'html'
				));
			}
		}
		
		$add_site_box = new Directly_Import_Meta_Box(array
		(
			'context' => 'normal',
			'id' => 'import',
			'title' => __('Add Site', 'directly-import')
		));
		
		$add_site_box->add_field(array
		(
			'description' => __('Enter the export URL for the site you\'d like to import from.', 'directly-import'),
			'input_classes' => array('diim-ignore-change'),
			'name' => 'add_site',
			'type' => 'text',
			
			'validation' => array
			(
				'required' => true
			)
		));
		
		$add_site_box->add_field(array
		(
			'content' => '<button type="button" class="button diim-button diim-ajax-button diim-add-site">' . __('Add Site', 'directly-import') . '</button>',
			'type' => 'html'
		));
		
		if (!empty($archived))
		{
			$archived_box = new Directly_Import_Meta_Box(array
			(
				'context' => 'normal',
				'id' => 'archived_sites',
				'title' => __('Archived Sites', 'directly-import')
			));
			
			$restore_site_label = __('Restore Site', 'directly-import');
			$delete_site_label = __('Delete Site', 'directly-import');
			
			foreach ($archived as $url)
			{
				$archived_box->add_field(array
				(
					'type' => 'html',
					
					'content' => '<p class="diim-text-large">' . $url . '</p>'
					. '<p><button type="button" class="button diim-button diim-ajax-button diim-restore-site" data-diim-url="' . esc_attr($url) . '">' . $restore_site_label . '</button> '
					. '<button type="button" class="button diim-button diim-ajax-button diim-delete-site" data-diim-url="' . esc_attr($url) . '">' . $delete_site_label . '</button></p>'
				));
			}
		}
		
		Directly_Import_Meta_Box::side_meta_boxes();
		Directly_Import_Meta_Box::finalize_meta_boxes();
	}
}
