<?php
/*!
 * Meta box field functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Field
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the meta box object.
 *
 * @since 1.0.0
 *
 * @uses Directly_Import_Wrapper
 */
final class Directly_Import_Field extends Directly_Import_Wrapper
{
	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  array $options Options for the field.
	 * @return void
	 */
	public function __construct($options)
	{
		parent::__construct();

		$this->_set_properties($options);

		if
		(
			!empty($this->name)
			&&
			is_array($this->validation)
			&&
			!empty($this->validation)
		)
		{
			$this->base->cache->_push('form_validation', $this->validation, $this->id);
		}
	}
	
	/**
	 * Get a default value based on the provided name.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 * @param  string $name Name of the value to return.
	 * @return mixed        Default value if it exists, otherwise an empty string.
	 */
	protected function _default($name)
	{
		switch ($name)
		{
			/**
			 * Custom value for a checkbox field.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'checkbox_value':
			
			/**
			 * Content added to the field.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'content':
			
			/**
			 * Short description display with the field.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'description':
			
			/**
			 * Output label displayed with the field.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'label':
			
			/**
			 * Base name for the field.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'name':
			
			/**
			 * Meta box option name.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'option_name':
			
			/**
			 * Current value for the field.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'value':
			
				return '';

			/**
			 * Additional attributes to add to the field.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'attributes':
			
			/**
			 * CSS classes added to the field wrapper.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'classes':
			
			/**
			 * Conditions for a field to be visible.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'conditional':
			
			/**
			 * Attributes for the main field wrapper.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'field_attributes':
			
			/**
			 * CSS classes added to the field input element.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'input_classes':
			
			/**
			 * Field options. Only works with Select fields.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'options':
			
			/**
			 * Validation options for the field.
			 *
			 * @since 1.0.0
			 *
			 * @var array
			 */
			case 'validation':
			
				return array();

			/**
			 * True if the labels should be hidden from the field output.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'hide_labels':
			
			/**
			 * True if a clear DIV should be added after this field.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'include_clear':
			
			/**
			 * True if the field is tall and the description should be displayed below the label.
			 *
			 * @since 1.0.0
			 *
			 * @var boolean
			 */
			case 'is_tall':
			
				return false;

			/**
			 * Type of field to output.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'type':
			
				return 'text';
				
			/**
			 * Generated DOM ID.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'id':

				return $this->_generate_id();

			/**
			 * Generated field identifier attributes.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'input_attributes':

				$attributes = '';
				
				if (!empty($this->id))
				{
					$attributes = (strpos($this->id, '[__i__]') === false)
					? ' id="diim-' . $this->id . '" name="' . $this->id . '"'
					: ' data-diim-identifier="' . $this->id . '"';
				}
				
				if ($this->type != 'image')
				{
					$attributes .= $this->_general_attributes();
				}

				return $attributes;

			/**
			 * Generated label attributes.
			 *
			 * @since 1.0.0
			 *
			 * @var string
			 */
			case 'label_attribute':

				if (empty($this->id))
				{
					return '';
				}
				
				return (strpos($this->id, '[__i__]') === false)
				? ' for="diim-' . $this->id . '"'
				: ' data-diim-identifier="diim-' . $this->id . '"';
		}

		return parent::_default($name);
	}

	/**
	 * Generate the output for the field.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  boolean     $echo True if the field should be echoed.
	 * @return string/void       Generated field if $echo is false.
	 */
	public function output($echo = false)
	{
		$this->_push('classes', 'diim-field');
		$this->_push('classes', 'diim-field-' . str_replace('_', '-', $this->type));
		
		$output = $this->_main_output();

		if
		(
			!empty($output)
			&&
			in_array('diim-hidden', $this->classes)
		)
		{
			$output .= '<div class="diim-hidden diim-field-spacer"></div>';
		}

		if (!$echo)
		{
			return $output;
		}

		echo $output;
	}

	/**
	 * Main field output.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  string $field            Current field output.
	 * @param  string $field_attributes Additional attributes added to the field wrapper.
	 * @return string                   Generated field.
	 */
	private function _main_output($field = '', $field_attributes = '')
	{
		$output = '';

		if (empty($field))
		{
			$type = (method_exists($this, '_field_' . $this->type))
			? $this->type
			: 'text';
			
			$field = call_user_func(array($this, '_field_' . $type));
		}

		if (!empty($field))
		{
			if (is_array($this->field_attributes))
			{
				foreach ($this->field_attributes as $name => $value)
				{
					$field_attributes .= ' ' . sanitize_key($name) . '="' . esc_attr($value) . '"';
				}
			}
			
			$field .= $this->_generate_condition_fields();
			$label_description = '';

			$description =
			(
				$this->hide_labels
				||
				empty($this->description)
			)
			? ''
			: '<div class="diim-description">'
			. '<label' . $this->label_attribute . '>' . $this->description . '</label>'
			. '</div>';

			if ($this->is_tall)
			{
				$label_description = $description;
				$description = '';
			}

			$label =
			(
				$this->hide_labels
				||
				empty($this->label)
			)
			? ''
			: '<div class="diim-field-label">'
			. $this->_generate_label()
			. $label_description
			. '</div>';

			$output = '<div class="' . esc_attr(implode(' ', $this->classes)) . '"' . $field_attributes . '>'
			. $label
			. '<div class="diim-field-input">'
			. $field
			. $description
			. '</div>'
			. '</div>';
			
			if ($this->include_clear)
			{
				$output .= '<div class="diim-clear"></div>';
			}
		}

		return $output;
	}

	/**
	 * Generate a checkbox field.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated checkbox field.
	 */
	private function _field_checkbox()
	{
		if (empty($this->id))
		{
			return '';
		}
		
		if ($this->checkbox_value === '')
		{
			$this->checkbox_value = '1';
		}
		
		if ($this->value === true)
		{
			$this->value = '1';
		}
		else if (is_array($this->value))
		{
			$this->value = in_array($this->checkbox_value, $this->value)
			? $this->checkbox_value
			: '';
		}

		$checkbox = '<input' . $this->input_attributes . ' type="checkbox" value="' . esc_attr($this->checkbox_value) . '"' . $this->get_input_classes() . ' ' . checked($this->checkbox_value, $this->value, false) . ' />';

		if
		(
			is_array($this->attributes)
			&&
			isset($this->attributes['placeholder'])
		)
		{
			$label_attr =
			(
				!$this->hide_labels
				||
				empty($this->description)
			)
			? ''
			: ' diim-tooltip" data-diim-tooltip="' . esc_attr($this->description);
			
			$checkbox = '<label class="diim-description' . $label_attr . '">' . $checkbox . '<span>' . $this->attributes['placeholder'] . '</span></label>';
		}

		return $checkbox;
	}

	/**
	 * Generate an HTML field.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated HMTL field.
	 */
	private function _field_html()
	{
		return '<div class="diim-html' . $this->get_input_classes(false) . '">'
		. wpautop(do_shortcode($this->content))
		. '</div>';
	}

	/**
	 * Generate a select field.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated select field.
	 */
	private function _field_select()
	{
		if (empty($this->id))
		{
			return '';
		}

		$field = '<select' . $this->input_attributes . $this->get_input_classes() . '>';
		
		if (!empty($this->options))
		{
			foreach ($this->options as $value => $label)
			{
				if (is_array($label))
				{
					$field .= '<optgroup label="' . esc_attr($value) . '">';
					
					foreach ($label as $group_value => $group_label)
					{
						$field .= '<option value="' . esc_attr($group_value) . '" ' . selected($this->value, $group_value, false) . '>' . $group_label . '</option>';
					}
					
					$field .= '</optgroup>';
				}
				else
				{
					$field .= '<option value="' . esc_attr($value) . '" ' . selected($this->value, $value, false) . '>' . $label . '</option>';
				}
			}
		}

		$field .= '</select>';

		return $field;
	}

	/**
	 * Generate a submit button.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated submit button.
	 */
	private function _field_submit()
	{
		$this->content = (empty($this->content))
		? __('Submit', 'directly-import')
		: $this->content;

		return '<button' . $this->input_attributes . ' type="submit" disabled="disabled" class="button button-large button-primary diim-button' . $this->get_input_classes(false) . '"><span>' . $this->content . '</span></button>';
	}

	/**
	 * Generate a text field.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated text field.
	 */
	private function _field_text()
	{
		if (empty($this->id))
		{
			return '';
		}
		
		if (is_array($this->value))
		{
			$this->value = '';
		}

		$input_type = esc_attr($this->type);
		$max_width_open = $max_width_close = '';

		if
		(
			is_array($this->attributes)
			&&
			isset($this->attributes['maxlength'])
			&&
			is_numeric($this->attributes['maxlength'])
		)
		{
			$max_width_open = '<div style="max-width: ' . ($this->attributes['maxlength'] * 14 + 32) . 'px;">';
			$max_width_close = '</div>';
		}

		$this->type = 'text';

		return $max_width_open
		. '<input' . $this->input_attributes . ' type="' . $input_type . '" value="' . esc_attr($this->value) . '"' . $this->get_input_classes() . ' />'
		. $max_width_close;
	}

	/**
	 * Generate a textarea field.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated textarea field.
	 */
	private function _field_textarea()
	{
		return (empty($this->id))
		? ''
		: '<textarea' . $this->input_attributes . $this->get_input_classes() . '>' . esc_textarea($this->value) . '</textarea>';
	}

	/**
	 * Get the input class(es).
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @param  boolean $add_attr True if the class attribute should be added.
	 * @return string            Generated field class(es).
	 */
	public function get_input_classes($add_attr = true)
	{
		if ($this->is_template)
		{
			$this->_push('input_classes', 'diim-input-template');
		}

		if (!empty($this->input_classes))
		{
			$classes = Directly_Import_Utilities::check_array($this->input_classes);
			$classes = esc_attr(implode(' ', $classes));

			return ($add_attr)
			? ' class="' . $classes . '"'
			: ' ' . $classes;
		}

		return '';
	}

	/**
	 * Assemble the general attributes.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Assembled attributes.
	 */
	private function _general_attributes()
	{
		$is_checkbox = ($this->type == 'checkbox');
		$raw_attributes = Directly_Import_Utilities::check_array($this->attributes);
		$attributes = '';

		foreach ($raw_attributes as $name => $value)
		{
			if ($value != '')
			{
				if
				(
					$this->type == 'select'
					&&
					$name == 'placeholder'
				)
				{
					$attributes .= ' data-allow-clear="true" data-placeholder="' . esc_attr($value) . '"';
				}
				else if
				(
					!$is_checkbox
					||
					$name != 'placeholder'
				)
				{
					$attributes .= ' ' . sanitize_key($name) . '="' . esc_attr($value) . '"';
				}
			}
		}

		return $attributes;
	}

	/**
	 * Generate contition fields.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated condition fields.
	 */
	private function _generate_condition_fields()
	{
		$output = '';

		if
		(
			is_array($this->conditional)
			&&
			!empty($this->conditional)
		)
		{
			foreach ($this->conditional as $condition)
			{
				if
				(
					is_array($condition)
					&&
					isset($condition['field'])
					&&
					isset($condition['value'])
				)
				{
					if (!isset($condition['compare']))
					{
						$condition['compare'] = '=';
					}

					$output .= '<div class="diim-hidden diim-condition" '
					. 'data-diim-conditional="' . $this->id . '" '
					. 'data-diim-field="' . esc_attr($this->_generate_id($condition['field'])) . '" '
					. 'data-diim-value="' . esc_attr($condition['value']) . '" '
					. 'data-diim-compare="' . esc_attr($condition['compare']) . '">'
					. '</div>';
				}
			}
		}

		return $output;
	}

	/**
	 * Generate a field ID.
	 *  
	 * @since 1.0.0
	 *
	 * @access private
	 * @param  string $name The base name for the field. If excluded the default field name will be used.
	 * @return string       Generated field ID.
	 */
	private function _generate_id($name = '')
	{
		$name = (empty($name))
		? $this->name
		: $name;

		return
		(
			empty($name)
			||
			empty($this->option_name)
		)
		? $name
		: $this->option_name . str_replace('[]]', '][]', '[' . $name . ']');
	}
	
	/**
	 * Generate the label output.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @return string Generated label output.
	 */
	private function _generate_label()
	{
		if
		(
			empty($this->validation)
			&&
			is_array($this->input_classes)
			&&
			in_array('required', $this->input_classes)
		)
		{
			$this->validation = true;
		}
		
		return (empty($this->label))
		? ''
		: '<label' . $this->label_attribute . '><strong>' . Directly_Import_Output::required_asterisk($this->label, $this->validation) . '</strong></label>';
	}
}
