<?php
/*!
 * Plugin output functionality.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Output
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement the output functionality.
 *
 * @since 1.0.0
 */
final class Directly_Import_Output
{
	/**
	 * Admin notices to display on the admin page.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @var    string
	 */
	public static $admin_notices = '';

	/**
	 * Admin page tabs.
	 *
	 * @since 1.0.0
	 *
	 * @access private static
	 * @var    array
	 */
	private static $_tabs = array();

	/**
	 * Generate and add an admin notice.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string       $message        Message to display in the admin notice.
	 * @param  array/string $class          Optional CSS class(es) to add to the admin notice.
	 * @param  boolean      $is_dismissible True if the admin notice should be dismissible.
	 * @return void
	 */
	public static function add_admin_notice($message, $class = array(Directly_Import_Constants::NOTICE_SUCCESS), $is_dismissible = true)
	{
		self::$admin_notices .= self::admin_notice($message, $class, $is_dismissible);
	}

	/**
	 * Add an admin page tab.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string $menu_parent Parent page for the admin page.
	 * @param  string $menu_slug   Menu slug for the admin page.
	 * @param  string $page_title  Title for the admin page tab.
	 * @return void
	 */
	public static function add_tab($menu_parent, $menu_slug, $title)
	{
		self::$_tabs[] = array
		(
			'menu_parent' => $menu_parent,
			'menu_slug' => $menu_slug,
			'title' => $title
		);
	}

	/**
	 * Output an admin form page.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string $heading     Heading displayed at the top of the admin form page.
	 * @param  string $option_name Option name to generate the admin form page for.
	 * @return void
	 */
	public static function admin_form_page($heading, $option_name = '')
	{
		$diim = Directly_Import();
		
		self::$admin_notices = self::admin_notice(__('Please correct the error(s) below.', 'directly-import'), array(Directly_Import_Constants::NOTICE_ERROR, 'diim-hidden', 'diim-validation-error'), false)
		. self::$admin_notices;

		echo '<div class="wrap">';

		self::admin_nav_bar($heading);
		
		if (!empty($option_name))
		{
			$option_name = sanitize_key($option_name);
		}

		$has_option_name = (!empty($option_name));
		$screen = $diim->cache->screen;
		$columns = $screen->get_columns();

		$action = ($has_option_name)
		? 'options.php'
		: basename($_SERVER['REQUEST_URI']);
		
		if (empty($columns))
		{
			$columns = 2;
		}

		echo '<form action="' . esc_url(admin_url($action)) . '" method="post" class="diim-form">';

		if ($has_option_name)
		{
			settings_fields($option_name);
		}

		wp_nonce_field('closedpostboxes', 'closedpostboxesnonce', false);
		wp_nonce_field('meta-box-order', 'meta-box-order-nonce', false);

		echo '<div id="poststuff">'
		. '<div id="post-body" class="metabox-holder columns-' . $columns . '">'
		. '<div id="postbox-container-1" class="postbox-container">';

		do_meta_boxes($screen->id, 'side', '');

		echo '</div>'
		. '<div id="diim-primary-wrapper">'
		. '<div id="postbox-container-2" class="postbox-container">';

		do_meta_boxes($screen->id, 'advanced', '');
		do_meta_boxes($screen->id, 'normal', '');

		echo '</div>'
		. '</div>'
		. '<div class="diim-clear"></div>'
		. '</div>'
		. '</div>'
		. '</form>'
		. '</div>';
	}

	/**
	 * Output the admin page nav bar.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string $heading Heading displayed in the nav bar.
	 * @return void
	 */
	public static function admin_nav_bar($heading)
	{
		$diim = Directly_Import();
		$buttons = '';
		
		if (isset($_GET['settings-updated']))
		{
			self::add_admin_notice(__('Settings saved.', 'directly-import'));
		}

		echo '<div class="diim-nav">'
		. '<div class="diim-nav-title">'
		. '<h1>'
		. '<strong>' . $diim->cache->plugin_data['Name'] . '</strong> | ' . $heading
		. '</h1>'
		. '<div class="diim-clear"></div>'
		. '</div>';
		
		if (!empty(self::$_tabs))
		{
			echo '<div class="diim-tab-wrapper">';

			foreach (self::$_tabs as $tab)
			{
				$active_class = ($diim->cache->current_page == $tab['menu_slug'])
				? ' diim-tab-active'
				: '';

				echo '<a href="' . esc_url(admin_url($tab['menu_parent'] . '?page=' . $tab['menu_slug'])) . '" class="diim-tab' . $active_class . '">' . $tab['title'] . '</a>';
			}

			echo '</div>';
		}
		
		echo '</div>'
		. '<hr class="wp-header-end" />'
		. self::$admin_notices;
	}

	/**
	 * Generate an admin notice.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string       $message        Message to display in the admin notice.
	 * @param  array/string $class          Optional CSS class(es) to add to the admin notice.
	 * @param  boolean      $is_dismissible True if the admin notice should be dismissible.
	 * @return string                       Generated admin notice.
	 */
	public static function admin_notice($message, $class = array(Directly_Import_Constants::NOTICE_SUCCESS), $is_dismissible = true)
	{
		$classes = array();
		
		if (is_array($class))
		{
			$classes = $class;
		}
		else if (!empty($class))
		{
			$classes = explode(' ', $class);
		}

		if ($is_dismissible)
		{
			$classes[] = 'is-dismissible';
		}

		$admin_notice = '<div class="notice diim-notice';
		
		if (!empty($classes))
		{
			$admin_notice .= ' ' . esc_attr(implode(' ', $classes));
		}

		$admin_notice .= '">'
		. wpautop('<strong>' . $message . '</strong>')
		. '</div>';

		return $admin_notice;
	}

	/**
	 * Add a required asterisk to a label if necessary.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string  $label      Label to update with an asterisk.
	 * @param  array   $validation Array of validation rules for the object.
	 * @param  boolean $add_class  True if the asterisk should have the required class added.
	 * @return string              Modified label.
	 */
	public static function required_asterisk($label, $validation, $add_class = true)
	{
		if
		(
			$validation === true
			||
			(
				is_array($validation)
				&&
				isset($validation['required'])
				&&
				$validation['required']
			)
		)
		{
			$label = sprintf
			(
				_x('%1$s *', 'Required Field Label', 'directly-import'),
				$label
			);
		}

		return ($add_class)
		? str_replace('*', '<span class="diim-required">*</span>', $label)
		: $label;
	}
}
