<?php
/*!
 * Plugin utility functions.
 *
 * @since 1.0.0
 *
 * @package    Directly Import
 * @subpackage Utilities
 */

if (!defined('ABSPATH'))
{
	exit;
}

/**
 * Class used to implement utility functions.
 *
 * @since 1.0.0
 */
final class Directly_Import_Utilities
{
	/**
	 * Check a value to see if it is an array or convert to an array if necessary.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  mixed $value        Value to turn into an array.
	 * @param  mixed $return_false True if a false value should be returned as-is.
	 * @return array               Checked value as an array.
	 */
	public static function check_array($value, $return_false = false)
	{
		if
		(
			$value === false
			&&
			$return_false
		)
		{
			return $value;
		}

		if (empty($value))
		{
			$value = array();
		}

		if (!is_array($value))
		{
			$value = array($value);
		}

		return $value;
	}

	/**
	 * Remove comments, line breaks and tabs from provided code.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string $code Raw code to clean up.
	 * @return string       Code without comments, line breaks and tabs.
	 */
	public static function clean_code($code)
	{
		$code = preg_replace('/<!--(.*)-->/Uis', '', $code);
		$code = preg_replace('/(?:(?:\/\*(?:[^*]|(?:\*+[^*\/]))*\*+\/)|(?:(?<!\:|\\\|\'|\")\/\/.*))/', '', $code);

		return str_replace(array(PHP_EOL, "\r", "\n", "\t"), '', $code);
	}
	
	/**
	 * Check to see if a full string end with a specified string.
	 * 
	 * @since 1.0.0
	 * 
	 * @access public static
	 * @param  string  $needle   String to check for.
	 * @param  string  $haystack Full string to check.
	 * @return boolean           True if the full string ends with the specified string.
	 */
	public static function ends_with($needle, $haystack)
	{
		$length = strlen($needle);
		
		if ($length == 0)
		{
			return true;
		}
		
		return (substr($haystack, -$length) === $needle);
	}
	
	/**
	 * Get a response from an export URL.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string $url Escaped URL to get the response from.
	 * @return array       Full response array.
	 */
	public static function get_response($url)
	{
		$response = (empty($url))
		? ''
		: wp_safe_remote_get($url);
		
		$failure = array
		(
			'success' => false
		);
		
		if
		(
			empty($response)
			||
			!is_array($response)
			||
			!isset($response['body'])
			||
			empty($response['body'])
		)
		{
			return $failure;
		}
		
		$body = json_decode($response['body'], true);
		
		return (json_last_error() == JSON_ERROR_NONE)
		? $body
		: $failure;
	}

	/**
	 * Check to see if a plugin is active.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string  $path Path for the plugin to check.
	 * @return boolean       True if the plugin is active.
	 */
	public static function is_plugin_active($path)
	{
		if (!function_exists('is_plugin_active'))
		{
			require_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		return
		(
			is_plugin_active($path)
			&&
			file_exists(WP_PLUGIN_DIR . '/' . $path)
		);
	}

	/**
	 * Load and decode JSON from a provided file path.
	 *
	 * @since 1.0.0
	 *
	 * @access public static
	 * @param  string $file_path   Path to the JSON file.
	 * @param  string $plugin_path Path for the current plugin.
	 * @return string              Decoded JSON file.
	 */
	public static function load_json($file_path, $plugin_path = '')
	{
		if (empty($plugin_path))
		{
			$plugin_path = Directly_Import()->plugin;
		}
		
		$file = plugin_dir_path($plugin_path) . $file_path;

		if (!file_exists($file))
		{
			return '';
		}

		ob_start();

		require($file);

		return json_decode(ob_get_clean(), true);
	}
	
	/**
	 * Check to see if a full string starts with a specified string.
	 * 
	 * @since 1.0.0
	 * 
	 * @access public static
	 * @param  string  $needle   String to check for.
	 * @param  string  $haystack Full string to check.
	 * @return boolean           True if the full string starts with the specified string.
	 */
	public static function starts_with($needle, $haystack)
	{
		return
		(
			empty($needle)
			||
			strpos($haystack, $needle) === 0
		);
	}
}
